/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * 
 *   http://www.apache.org/licenses/LICENSE-2.0
 *   
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

/* $Rev: 453499 $ $Date: 2006-10-06 06:07:31 +0100 (Fri, 06 Oct 2006) $ */

#ifndef tuscany_sca_core_tuscanyruntime_h
#define tuscany_sca_core_tuscanyruntime_h

#include "tuscany/sca/export.h"
#include <string>
using std::string;

namespace tuscany
{
    namespace sca
    {
        
        /**
         * A singleton which represents the executing SCA runtime.
         */
        class SCA_API TuscanyRuntime 
        {
        public:
            /**
            * Default constructor
            */
            TuscanyRuntime(const string& defaultComponentName = "",
                const string& root = "", const string& path = "");            
            
            /**
            * Destructor
            */
            virtual ~TuscanyRuntime();            
            
            
            /**
            * Set the system root configuration path
            * @param root The path to the system configuration.
            */
            void setSystemRoot(const string& root);
            
            /**
            * Set the system composite search path
            * @param root The search path for composites.
            */
            void setSystemPath(const string& path);
            
            /**
            * Set the default component for the system
            * @param componentName The name of the default component.
            */
            void setDefaultComponentName(const string& componentName);
            
            /**
            * start the runtime
            */
            void start();       
            
            /**
            * stop the runtime
            */
            void stop();            
            
            
        private:
            string systemRoot;
            string systemPath;
            string defaultComponentName;
        };

        
    } // End namespace sca
} // End namespace tuscany




#endif // tuscany_sca_core_tuscanyruntime_h

